@@ -6,7 +6,7 @@
 from pathlib import Path
 from tempfile import TemporaryDirectory
 from textwrap import dedent
-from typing import Collection, Mapping, Optional, Set, Tuple
+from typing import Collection, Dict, Mapping, Optional, Set, Tuple
 
 import libcst as cst
 from libcst import ensure_type
@@ -19,7 +19,7 @@ from libcst.metadata import (
 )
 from libcst.metadata.full_repo_manager import FullRepoManager
 from libcst.metadata.name_provider import FullyQualifiedNameVisitor
-from libcst.testing.utils import UnitTest
+from libcst.testing.utils import data_provider, UnitTest
 
 
 def get_qualified_name_metadata_provider(
@@ -102,7 +102,8 @@ class QualifiedNameProviderTest(UnitTest):
         cls = ensure_type(m.body[1], cst.ClassDef)
         f = ensure_type(cls.body.body[0], cst.FunctionDef)
         self.assertEqual(
-            names[ensure_type(f.returns, cst.Annotation).annotation], set()
+            names[ensure_type(f.returns, cst.Annotation).annotation],
+            {QualifiedName("a.b.c", QualifiedNameSource.IMPORT)},
         )
 
         c_call = ensure_type(
@@ -414,67 +415,118 @@ class QualifiedNameProviderTest(UnitTest):
             },
         )
 
+    def test_shadowed_assignments(self) -> None:
+        m, names = get_qualified_name_metadata_provider(
+            """
+                from lib import a,b,c
+                a = a
+                class Test:
+                    b = b
+                def func():
+                    c = c
+            """
+        )
+
+        # pyre-fixme[53]: Captured variable `names` is not annotated.
+        def test_name(node: cst.CSTNode, qnames: Set[QualifiedName]) -> None:
+            name = ensure_type(
+                ensure_type(node, cst.SimpleStatementLine).body[0], cst.Assign
+            ).value
+            self.assertEqual(names[name], qnames)
+
+        test_name(m.body[1], {QualifiedName("lib.a", QualifiedNameSource.IMPORT)})
+
+        cls = ensure_type(m.body[2], cst.ClassDef)
+        test_name(
+            cls.body.body[0], {QualifiedName("lib.b", QualifiedNameSource.IMPORT)}
+        )
+
+        func = ensure_type(m.body[3], cst.FunctionDef)
+        test_name(
+            func.body.body[0], {QualifiedName("lib.c", QualifiedNameSource.IMPORT)}
+        )
+
 
 class FullyQualifiedNameProviderTest(UnitTest):
-    def test_builtins(self) -> None:
-        qnames = get_fully_qualified_names(
-            "test/module.py",
-            """
-            int(None)
-            """,
+    @data_provider(
+        (
+            # test module names
+            ("a/b/c.py", "", {"a.b.c": QualifiedNameSource.LOCAL}),
+            ("a/b.py", "", {"a.b": QualifiedNameSource.LOCAL}),
+            ("a.py", "", {"a": QualifiedNameSource.LOCAL}),
+            ("a/b/__init__.py", "", {"a.b": QualifiedNameSource.LOCAL}),
+            ("a/b/__main__.py", "", {"a.b": QualifiedNameSource.LOCAL}),
+            # test builtinxsx
+            (
+                "test/module.py",
+                "int(None)",
+                {
+                    "test.module": QualifiedNameSource.LOCAL,
+                    "builtins.int": QualifiedNameSource.BUILTIN,
+                    "builtins.None": QualifiedNameSource.BUILTIN,
+                },
+            ),
+            # test imports
+            (
+                "some/test/module.py",
+                """
+                from a.b import c as d
+                from . import rel
+                from .lol import rel2
+                from .. import thing as rel3
+                d, rel, rel2, rel3
+                """,
+                {
+                    "some.test.module": QualifiedNameSource.LOCAL,
+                    "a.b.c": QualifiedNameSource.IMPORT,
+                    "some.test.rel": QualifiedNameSource.IMPORT,
+                    "some.test.lol.rel2": QualifiedNameSource.IMPORT,
+                    "some.thing": QualifiedNameSource.IMPORT,
+                },
+            ),
+            # test more imports
+            (
+                "some/test/module/__init__.py",
+                """
+                from . import rel
+                from .lol import rel2
+                rel, rel2
+                """,
+                {
+                    "some.test.module": QualifiedNameSource.LOCAL,
+                    "some.test.module.rel": QualifiedNameSource.IMPORT,
+                    "some.test.module.lol.rel2": QualifiedNameSource.IMPORT,
+                },
+            ),
+            # test locals
+            (
+                "some/test/module.py",
+                """
+                class X:
+                    a: X
+                """,
+                {
+                    "some.test.module": QualifiedNameSource.LOCAL,
+                    "some.test.module.X": QualifiedNameSource.LOCAL,
+                    "some.test.module.X.a": QualifiedNameSource.LOCAL,
+                },
+            ),
         )
-        module_name = QualifiedName(
-            name="test.module", source=QualifiedNameSource.LOCAL
-        )
-        self.assertIn(module_name, qnames)
-        qnames -= {module_name}
-        self.assertEqual(
-            {"builtins.int", "builtins.None"},
+    )
+    def test_qnames(
+        self, file: str, code: str, names: Dict[str, QualifiedNameSource]
+    ) -> None:
+        qnames = get_fully_qualified_names(file, code)
+        self.assertSetEqual(
+            set(names.keys()),
             {qname.name for qname in qnames},
         )
         for qname in qnames:
-            self.assertEqual(qname.source, QualifiedNameSource.BUILTIN, msg=f"{qname}")
-
-    def test_imports(self) -> None:
-        qnames = get_fully_qualified_names(
-            "some/test/module.py",
-            """
-            from a.b import c as d
-            from . import rel
-            from .lol import rel2
-            from .. import thing as rel3
-            d, rel, rel2, rel3
-            """,
-        )
-        module_name = QualifiedName(
-            name="some.test.module", source=QualifiedNameSource.LOCAL
-        )
-        self.assertIn(module_name, qnames)
-        qnames -= {module_name}
-        self.assertEqual(
-            {"a.b.c", "some.test.rel", "some.test.lol.rel2", "some.thing"},
-            {qname.name for qname in qnames},
-        )
-        for qname in qnames:
-            self.assertEqual(qname.source, QualifiedNameSource.IMPORT, msg=f"{qname}")
-
-    def test_locals(self) -> None:
-        qnames = get_fully_qualified_names(
-            "some/test/module.py",
-            """
-            class X:
-                a: X
-            """,
-        )
-        self.assertEqual(
-            {"some.test.module", "some.test.module.X", "some.test.module.X.a"},
-            {qname.name for qname in qnames},
-        )
-        for qname in qnames:
-            self.assertEqual(qname.source, QualifiedNameSource.LOCAL, msg=f"{qname}")
+            self.assertEqual(qname.source, names[qname.name], msg=f"{qname}")
 
     def test_local_qualification(self) -> None:
-        base_module = "some.test.module"
+        module_name = "some.test.module"
+        package_name = "some.test"
         for (name, expected) in [
             (".foo", "some.test.foo"),
             ("..bar", "some.bar"),
@@ -483,8 +535,7 @@ class FullyQualifiedNameProviderTest(UnitTest):
             with self.subTest(name=name):
                 self.assertEqual(
                     FullyQualifiedNameVisitor._fully_qualify_local(
-                        base_module,
-                        QualifiedName(name=name, source=QualifiedNameSource.LOCAL),
+                        module_name, package_name, name
                     ),
                     expected,
                 )
